#ifndef __STRMANIP_H
#define __STRMANIP_H

#include <string>
#include <sstream>
#include <list>
#include <set>
#include <memory>
#include <ctype.h>

using namespace std;

inline string itos(int i)
{
    ostringstream str;
    str << i;
    return str.str();
};

inline string string_tolower(string s)
{
    for (string::iterator i = s.begin(); i != s.end(); i++)
       *i = tolower(*i);

    return s;
};

inline string string_brfilter(string s)
{
    int brackets = 0;
    string news = "";
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        char c = *i;
        if (c == '[' || c == '{' || c == '(')
        {
            brackets++;
            // Replace brackets with a "/" to act as a separator
            news += "/";
        }
        if (!brackets)
        {
            news += *i;
        }
        if ((c == ']' || c == '}' || c == ')') && brackets > 0)
        {
            brackets--;
        }
    }
    return news;
};            

inline string string_replace(string s, char c, char r)
{
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        if (*i == c)
            *i = r;
    }
    return s;
};            

inline string string_normalize(string s)
{
    s = string_brfilter(string_tolower(string_replace(s, '_', ' ')));
    string news = "";
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        if (islower(*i))
            news += *i;
    }
    return news;
};            

inline void string_chr_split(set<string> &store, const string &s,
        string delim = "\t\r\n ", string anti = "\"")
{
    unsigned int start = 0, last = 0, next;
    string sep = delim + anti;
    bool ignore = false;

    while ((next = s.find_first_of(sep, last)) != string::npos)
    {
        last = next + 1;

        if (anti.find(s[next]) != string::npos)
        {
            ignore = !ignore;
        }
        else if (!ignore)
        {
            if (start < next)
                store.insert(s.substr(start, next - start));
            start = last;
        }
    }

    if (start < s.length())
        store.insert(s.substr(start, s.length()));

    store.erase("");
};

inline list<string> &delimlst()
{
    static list<string> *delimlist = NULL;
    if (!delimlist)
    {
        delimlist = new list<string>;
        delimlist->push_back(" -");
        delimlist->push_back("- ");
        delimlist->push_back(". ");
        delimlist->push_back(" .");
    }
    return *delimlist;
}; 

inline void string_str_split(set<string> &store, const string &s,
        const list<string> &delims = delimlst())
{
    size_t start = 0;

    while (1)
    {
        size_t next = 10000;
        string next_del = "";
        for (list<string>::const_iterator i = delims.begin(); i != delims.end(); i++)
        {
            size_t pos = s.find(*i, start);
            if (pos < next)
            {
                next = pos;
                next_del = *i;
            }
        }
        if (next == 10000)
        {
            store.insert(string_normalize(s.substr(start)));
            return;
        }
        else
        {
            store.insert(string_normalize(s.substr(start, next - start)));
            start = next + next_del.length();
        }
    }
};

inline int string_count(const string &s, char c)
{
    int count = 0;
    for (string::const_iterator i = s.begin(); i != s.end(); i++)
    {
        count += (*i == c);
    }

    return count;
};

inline void imms_magic_parse(set<string> &store, string s)
{
    int unders = string_count(s, '_');
    int dashes = string_count(s, '-');
    int spaces = string_count(s, ' ');

    if (!spaces && dashes > 1 && unders > 1)
    {
        size_t lastdash = s.rfind("-");
        if (lastdash + 4 == s.length() || lastdash + 3 == s.length())
            s = s.substr(0, lastdash);
        size_t sep = s.find("_-");
        if (sep == string::npos)
            s = string_replace(s, '-', '/');
    }

    s = string_replace(s, '_', ' ');
    
    set<string> path_chunks;

    s = string_brfilter(s);
    string_chr_split(path_chunks, s, "/");

    for (set<string>::iterator i = path_chunks.begin();
            i != path_chunks.end(); i++)
    {
        string_str_split(store, *i);
    }  
    store.erase("");
};

#endif
